<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\DependencyInjection;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Customer\CustomerChannelPricing;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Customer\CustomerChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\CustomerGroup\CustomerGroupChannelPricing;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\CustomerGroup\CustomerGroupChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Organization\OrganizationChannelPricing;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\Organization\OrganizationChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\Customer\Organization;
use Dedi\SyliusCustomerPricingPlugin\Entity\Customer\OrganizationInterface;
use Dedi\SyliusCustomerPricingPlugin\Form\Type\Admin\OrganizationType;
use Dedi\SyliusCustomerPricingPlugin\Repository\OrganizationRepository;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Sylius\Component\Resource\Factory\Factory;
use Symfony\Component\Config\Definition\Builder\ArrayNodeDefinition;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    /**
     * @psalm-suppress UnusedVariable
     */
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_customer_pricing_plugin');
        $rootNode = $treeBuilder->getRootNode();

        $this->addOrganizationResourcesSection($rootNode);

        return $treeBuilder;
    }

    private function addOrganizationResourcesSection(ArrayNodeDefinition $node): void
    {
        $node
            ->children()
            ->arrayNode('resources')
                ->addDefaultsIfNotSet()
                ->children()
                    ->arrayNode('organization')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->variableNode('options')->end()
                            ->arrayNode('classes')
                                ->addDefaultsIfNotSet()
                                ->children()
                                    ->scalarNode('model')->defaultValue(Organization::class)->cannotBeEmpty()->end()
                                    ->scalarNode('interface')->defaultValue(OrganizationInterface::class)->cannotBeEmpty()->end()
                                    ->scalarNode('repository')->defaultValue(OrganizationRepository::class)->cannotBeEmpty()->end()
                                    ->scalarNode('factory')->defaultValue(Factory::class)->end()
                                    ->scalarNode('controller')->defaultValue(ResourceController::class)->end()
                                    ->scalarNode('form')->defaultValue(OrganizationType::class)->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                    ->arrayNode('customer_channel_pricing')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->variableNode('options')->end()
                            ->arrayNode('classes')
                                ->addDefaultsIfNotSet()
                                ->children()
                                    ->scalarNode('model')->defaultValue(CustomerChannelPricing::class)->cannotBeEmpty()->end()
                                    ->scalarNode('interface')->defaultValue(CustomerChannelPricingInterface::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('repository')->defaultValue(OrganizationRepository::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('factory')->defaultValue(Factory::class)->end()
//                                    ->scalarNode('controller')->defaultValue(ResourceController::class)->end()
//                                    ->scalarNode('form')->defaultValue(OrganizationType::class)->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                    ->arrayNode('customer_group_channel_pricing')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->variableNode('options')->end()
                            ->arrayNode('classes')
                                ->addDefaultsIfNotSet()
                                ->children()
                                    ->scalarNode('model')->defaultValue(CustomerGroupChannelPricing::class)->cannotBeEmpty()->end()
                                    ->scalarNode('interface')->defaultValue(CustomerGroupChannelPricingInterface::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('repository')->defaultValue(OrganizationRepository::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('factory')->defaultValue(Factory::class)->end()
//                                    ->scalarNode('controller')->defaultValue(ResourceController::class)->end()
//                                    ->scalarNode('form')->defaultValue(OrganizationType::class)->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                    ->arrayNode('oganization_channel_pricing')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->variableNode('options')->end()
                            ->arrayNode('classes')
                                ->addDefaultsIfNotSet()
                                ->children()
                                    ->scalarNode('model')->defaultValue(OrganizationChannelPricing::class)->cannotBeEmpty()->end()
                                    ->scalarNode('interface')->defaultValue(OrganizationChannelPricingInterface::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('repository')->defaultValue(OrganizationRepository::class)->cannotBeEmpty()->end()
//                                    ->scalarNode('factory')->defaultValue(Factory::class)->end()
//                                    ->scalarNode('controller')->defaultValue(ResourceController::class)->end()
//                                    ->scalarNode('form')->defaultValue(OrganizationType::class)->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ->end()
        ;
    }
}
