<?php

declare(strict_types=1);

namespace Tests\Dedi\SyliusCustomerPricingPlugin\Application\src\Repository;

use Dedi\SyliusCustomerPricingPlugin\Entity\Customer\OrganizationInterface;
use Dedi\SyliusCustomerPricingPlugin\Enum\ChannelPricingType;
use Dedi\SyliusCustomerPricingPlugin\Loader\ValueObject\ChannelPricing;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface as DediSyliusCustomerPricingPluginChannelRepositoryInterface;
use Dedi\SyliusCustomerPricingPlugin\Repository\Trait\ChannelPricingRepositoryTrait;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\NativeQuery;
use Doctrine\ORM\Query\ResultSetMapping;
use Doctrine\ORM\QueryBuilder;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Sylius\Component\Customer\Model\CustomerGroupInterface;

class ChannelPricingRepository extends EntityRepository implements DediSyliusCustomerPricingPluginChannelRepositoryInterface
{
    use ChannelPricingRepositoryTrait;

    public function createListQueryBuilder(string $alias = 'o'): QueryBuilder
    {
        return $this->createQueryBuilder($alias);
    }

    public function findForProductVariant(ProductVariantInterface $productVariant, string $type): Collection
    {
        $query = $this->createBaseNativeQuery(<<<SQL
            SELECT scp.id, scp.channel_code
            FROM sylius_channel_pricing scp
                INNER JOIN sylius_product_variant spv ON spv.id = scp.product_variant_id
            WHERE spv.id = :productVariant AND scp.`type` = :type
            SQL
            , [
                'productVariant' => $productVariant,
                'type' => $type
            ]
        );

        return $this->getChannelPricingCollectionFromQuery($query);
    }


    public function findDefaultForProductVariant(ProductVariantInterface $productVariant): Collection
    {
        return $this->findForProductVariant($productVariant, ChannelPricingType::DEFAULT->value);
    }

    public function findForCustomerForProductVariant(ProductVariantInterface $productVariant, CustomerInterface $customer): Collection
    {
        $query = $this->createBaseNativeQuery(<<<SQL
            SELECT scp.id, scp.channel_code
            FROM sylius_channel_pricing scp
                INNER JOIN sylius_product_variant spv ON spv.id = scp.product_variant_id
                INNER JOIN dedi_customer_pricing_customer_channel_pricing dcpccp ON scp.id = dcpccp.id
            WHERE spv.id = :productVariant
              AND dcpccp.customer_id = :customer
              AND scp.`type` = :type
            SQL
            , [
                'productVariant' => $productVariant,
                'customer' => $customer,
                'type' => ChannelPricingType::CUSTOMER->value
            ]
        );

        return $this->getChannelPricingCollectionFromQuery($query);
    }

    public function findForOrganizationForProductVariant(ProductVariantInterface $productVariant, OrganizationInterface $organization): Collection
    {
        $query = $this->createBaseNativeQuery(<<<SQL
            SELECT scp.id, scp.channel_code
            FROM sylius_channel_pricing scp
                INNER JOIN sylius_product_variant spv ON spv.id = scp.product_variant_id
                INNER JOIN dedi_customer_pricing_organization_channel_pricing dcpocp ON scp.id = dcpocp.id
            WHERE spv.id = :productVariant
              AND dcpocp.organization_id = :organization
              AND scp.`type` = :type
            SQL
            , [
                'productVariant' => $productVariant,
                'organization' => $organization,
                'type' => ChannelPricingType::ORGANIZATION->value
            ]
        );

        return $this->getChannelPricingCollectionFromQuery($query);
    }

    public function findForCustomerGroupForProductVariant(ProductVariantInterface $productVariant, CustomerGroupInterface $customerGroup): Collection
    {
        $query = $this->createBaseNativeQuery(<<<SQL
            SELECT scp.id, scp.channel_code
            FROM sylius_channel_pricing scp
                INNER JOIN sylius_product_variant spv ON spv.id = scp.product_variant_id
                INNER JOIN dedi_customer_pricing_customer_group_channel_pricing dcpcgcp ON scp.id = dcpcgcp.id
            WHERE spv.id = :productVariant
              AND dcpcgcp.customer_group_id = :customerGroup
              AND scp.`type` = :type
            SQL
            , [
                'productVariant' => $productVariant,
                'customerGroup' => $customerGroup,
                'type' => ChannelPricingType::CUSTOMER_GROUP->value
            ]
        );

        return $this->getChannelPricingCollectionFromQuery($query);
    }

    private function createBaseNativeQuery(string $sql, array $parameters): NativeQuery
    {
        $em = $this->getEntityManager();

        $rsm = new ResultSetMapping();
        $rsm->addScalarResult('id', 'id', \Doctrine\DBAL\Types\Types::INTEGER);
        $rsm->addScalarResult('channel_code', 'channelCode', \Doctrine\DBAL\Types\Types::STRING);
        $rsm->newObjectMappings['id'] = [
            'className' => ChannelPricing::class,
            'objIndex' => 0,
            'argIndex' => 0
        ];
        $rsm->newObjectMappings['channel_code'] = [
            'className' => ChannelPricing::class,
            'objIndex' => 0,
            'argIndex' => 1
        ];
        $query = $em->createNativeQuery($sql, $rsm);

        foreach ($parameters as $key => $value) {
            $query->setParameter($key, $value);
        }

        return $query;
    }

    /**
     * @return Collection<string, ChannelPricing>
     */
    private function getChannelPricingCollectionFromQuery(NativeQuery $query): Collection
    {
        /** @var ChannelPricing[] $result */
        $result = $query->getResult();
        return array_reduce($result, static function (Collection $channelPricings, ChannelPricing $channelPricing) {
            $channelPricings->set($channelPricing->getChannelCode(), $channelPricing);
            return $channelPricings;
        }, new ArrayCollection());
    }
}
